//
//  GHPImageView.swift
//  Granja
//
//  Created by Santiago Pavón on 21/10/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import UIKit

enum GHPState {
    case HEN
    case EGG
    case CHICKEN
    case FRIED
    case ROAST
}

class GHPImageView: UIImageView {

    
    // Estado que indica la imagen a mostrar
    var ghpState: GHPState = .HEN {
        didSet {
            updateGHPStateImage()
        }
    }
    
    // Inicializador
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        updateGHPStateImage()
        
        // Necesito enterarme de los eventos del usuario
        userInteractionEnabled = true
        
        // Crear reconocedor de Tap para ir de gallina a huevo, a pollo, y a gallina
        let tapRec = UITapGestureRecognizer(target: self, action: "processTap:")
        tapRec.numberOfTapsRequired = 1
        addGestureRecognizer(tapRec)
        
        // Crear reconocedor de Long Press para freir huevo
        let lpRec = UILongPressGestureRecognizer(target: self, action: "processLongPress:")
        addGestureRecognizer(lpRec)
        
        // Crear reconocedor de Pan para mover imagen view
        let panRec = UIPanGestureRecognizer(target: self, action: "processPan:")
        addGestureRecognizer(panRec)
        
        // Crear reconocedor de Pinch para escalar imagen view
        let pinchRec = UIPinchGestureRecognizer(target: self, action: "processPinch:")
        addGestureRecognizer(pinchRec)
        
        // Crear reconocedor de Rotation para girar imagen view
        let rotationRec = UIRotationGestureRecognizer(target: self, action: "processRotation:")
        addGestureRecognizer(rotationRec)
    }
    
    
    required init(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    // En funcion del estado se usa una imagen distinta
    private func updateGHPStateImage() {
        
        switch ghpState {
        case .HEN:
            image = UIImage(named: "gallina")
        case .EGG:
            image = UIImage(named: "huevo")
        case .CHICKEN:
            image = UIImage(named: "pollo")
        case .FRIED:
            image = UIImage(named: "frito")
        case .ROAST:
            image = UIImage(named: "asado")
        }
        
        if image != nil {
            bounds.size = image!.size
        }
    }
    
    func processTap(sender: UITapGestureRecognizer) {
        
        switch ghpState {
        case .HEN:
            ghpState = .EGG
        case .EGG:
            ghpState = .CHICKEN
        case .CHICKEN:
            ghpState = .HEN
        case .FRIED:
            break
        case .ROAST:
            break
        }
    }
    
    func processLongPress(sender: UILongPressGestureRecognizer) {
        
        if sender.state != .Began { return }
        
        UIView.transitionWithView(self,
            duration: 0.5,
            options: .TransitionFlipFromLeft,
            animations: {
                switch self.ghpState {
                case .HEN:
                    self.ghpState = .ROAST
                case .EGG:
                    self.ghpState = .FRIED
                case .CHICKEN:
                    break
                case .FRIED:
                    break
                case .ROAST:
                    break
                }
            },
            completion: nil)
    }
    
    func processPinch(sender: UIPinchGestureRecognizer) {
        let factor = sender.scale
        
        transform = CGAffineTransformScale(transform, factor, factor)
        
        sender.scale = 1
    }
    
    
    func processRotation(sender: UIRotationGestureRecognizer) {
        let angle = sender.rotation
        
        transform = CGAffineTransformRotate(transform, angle)
        
        sender.rotation = 0
    }
    
    
    func processPan(sender: UIPanGestureRecognizer) {
        let pos = sender.translationInView(sender.view!)
        
        transform = CGAffineTransformTranslate(transform, pos.x, pos.y)
        
        sender.setTranslation(CGPointZero, inView: sender.view)
    }
    
}
